//---------------------------------------------------------------------------------------
// Визуализация данных на трехзначном 7-сегментном индикаторе
// leddd05 v. 2016.11.01.10.25
// OldBean
//---------------------------------------------------------------------------------------
#define COMMON_CATHODE // Если индикатор с общим анодом - необходимо закомментировать эту строку
//---------------------------------------------------------------------------------------
#ifdef COMMON_CATHODE // Для индикаторов с общим катодом
#define L_HIGH 1
#define L_LOW  0
#else                 // Для индикаторов с общим анодом
#define L_HIGH 0
#define L_LOW  1
#endif
//---------------------------------------------------------------------------------------
// Пины, к которым подключены сегменты индикатора аноды (или катоды для индикаторов с общими анодами)
static uint8_t segPins[7] = {12, 16, 5, 7, 8, 13, 4};
// Пины, к которым подключены общие катоды (или аноды, для индикаторов с общими анодами)
static uint8_t digPins[3] = {15, 14, 11};
static uint8_t digit[11] = {0b00111111, // Маски сегментов цифр 0, 1, 2 и т.д.
                            0b00000110,
                            0b01011011,
                            0b01001111,
                            0b01100110,
                            0b01101101,
                            0b01111101,
                            0b00000111,
                            0b01111111,
                            0b01101111,
                            0b00000000}; // "Цифра" 10 это пробел
static int d[3]; // Сотни, десятки и единицы показываемые на разрядах 7-сегментного индикатора
static uint8_t mask = 0x01; // Маска для анализа цифр
static int seg = 0; // Номер сегмента, обрабатываемого (подствечиваемого, если нужно) в данный момент
//static int prSeg = 6; // Предыдущий сегмент
static unsigned long dt = 3000; // Длительность (в мкс) подсветки каждой группы сегментов
static volatile unsigned long stt = 0; // Момент времени (в мкс) когда заканчивается подсветка текущей группы сегментов
static volatile uint8_t nextSegFlag = 1; // Флаг для перехода к подсветке следующей группы сегментов

static volatile int rms = 137; // Среднеквадратичное значение измеряемого напряжения
static uint8_t updateFlag = 1; // 1 - данные (цифры), выводимые на индикатор, нужно обновить

// Рабочие переменные для отладки
static volatile unsigned long stg = 0;
static volatile int cntr = 0;
//---------------------------------------------------------------------------------------
void setup() {
  for(int i = 0; i < 7; i++) { // Инициализация пинов, к которым подключены аноды индикатора (сегменты)
    pinMode(segPins[i], OUTPUT);
    digitalWrite(segPins[i], L_LOW);
  }
  for(int i = 0; i < 3; i++) { // Инициализация пинов, к которым подключены катоды индикатора (цифры)
    pinMode(digPins[i], OUTPUT);
    digitalWrite(digPins[i], L_HIGH);
  }
  unsigned long st = millis() + 1000;
  d[0] = 8; d[1] = 8; d[2] = 8; // Для демонстрации работоспособности индикатора покажем все сегменты разом
  while(millis() < st) {
      ShowSegments();
  }
  d[0] = 10; d[1] = 10; d[2] = 10; // Очистим индикатор
  ShowSegments();

  stg = micros() + dt;

  // Вспомогательный пин для исследования временных характеристик
  pinMode(9, OUTPUT); // 9 - это PB1
  digitalWrite(9, LOW);
}
//---------------------------------------------------------------------------------------
void loop() {  
  if((++cntr % 10000) == 0) {
    if(++rms > 999)
      rms = 0;
    updateFlag = 1; 
  }
  if(updateFlag) // Обновим цифры, выводимые на индикатор
    UpdateDigits(rms);

//  PORTB |= 0b00000010;
//  stg = (unsigned long)rms*rms;
//  PORTB &= 0b11111101;
// У-у-у... Возведение в квадрат чисел unsigned long занимает 3.72 мкс

  if(micros() >= stg) {
    stg = micros() + dt;
    PORTB |= 0b00000010;
    ShowSegments(); // Подсвечиваем очередную группу сегментов. Время отработки функции - 28 мкс.
    PORTB &= 0b11111101;
    
  }
}
//---------------------------------------------------------------------------------------
void UpdateDigits(int val) { // Подготовка данных (цифр) для визуализации
  d[2] = val/100;
  d[1] = (val - d[2]*100)/10;
  d[0] = val - d[2]*100 - d[1]*10;
  if(d[2] == 0) { // Не отображаем ведущие нули ежели что... ("Цифра" 10 - это "пустая" цифра, все ее сегменты потушены)
    d[2] = 10;
    if(d[1] == 0) d[1] = 10;
  }
  updateFlag = 0;
}
//---------------------------------------------------------------------------------------
void ShowSegments() { // Подсвечивание очередной группы сегментов
  digitalWrite(segPins[seg], L_LOW); // "Роняем" потенциал анода "отработавшего" сегмента
  if(++seg < 7) mask <<= 1; // Переходим к следующему сегменту
  else { // Дошли до последнего сегмента
    seg = 0; mask = 0x01;
  }
  digitalWrite(segPins[seg], L_HIGH); // "Подбрасываем" потенциал анода нового текущего сегмента
  for(int bi = 0; bi < 3; bi++) { // Теперь пробегаем по цифрам (разрядам) индикатора
    if(digit[d[bi]] & mask) // Если данный сегмент в bi-ом разряде должен гореть,
      digitalWrite(digPins[bi], L_LOW); // то уроним потенциал катода соответствующей цифры (разряда)
    else // А если гореть не должен, то
      digitalWrite(digPins[bi], L_HIGH); // потенциал катода bi-й цифры должен быть высокий 
  }
}

